@@ -9,12 +9,18 @@ import pbr.version
 
 def __version__(release=False):
     # a function because this is expensive
-    if release:
+    if release:  # appends .devNNN
         return pbr.version.VersionInfo(__name__).release_string()
-    else:
+    else:  # semver only
         return pbr.version.VersionInfo(__name__).version_string()
 
 
+def versionTuple(v=None):
+    if v is None:
+        v = __version__(True)
+    return tuple(int(x.lstrip("dev") or 0) for x in v.split("."))
+
+
 import os
 import os.path
 import sys
@@ -118,8 +124,8 @@ def initLogging(level=None, logfile=None):
 
     oldLevel = _logLevel
     if level is not None:
-        _logLevel = level
         _logHandler.setLevel(level)
+        _logLevel = _logHandler.level
     return _logHandler, oldLevel
 
 
