@@ -4,8 +4,8 @@ import six
 import collections
 import re
 import os
-from .support import Status, Defaults, ResourceChanges
-from .result import serializeValue, ChangeAware, Results, ResultsMap
+from .support import Status, Defaults, ResourceChanges, Priority, TopologyMap
+from .result import serializeValue, ChangeAware, Results, ResultsMap, getDigest
 from .util import (
     registerClass,
     lookupClass,
@@ -21,7 +21,7 @@ from .util import (
 )
 from . import merge
 from .eval import Ref, mapValue, RefContext
-from .runtime import RelationshipInstance
+from .runtime import RelationshipInstance, Operational
 from .yamlloader import yaml
 
 import logging
@@ -29,18 +29,74 @@ import logging
 logger = logging.getLogger("unfurl.task")
 
 
-class TaskRequest(object):
+class PlanRequest(object):
+    def __init__(self, target):
+        self.target = target
+
+
+class TaskRequest(PlanRequest):
     """
     Yield this to run a child task. (see :py:meth:`unfurl.configurator.TaskView.createSubTask`)
     """
 
-    def __init__(self, configSpec, resource, reason, persist=False, required=None):
+    def __init__(
+        self,
+        configSpec,
+        target,
+        reason,
+        persist=False,
+        required=None,
+        startState=None,
+    ):
+        super(TaskRequest, self).__init__(target)
         self.configSpec = configSpec
-        self.target = resource
         self.reason = reason
         self.persist = persist
         self.required = required
         self.error = configSpec.name == "#error"
+        self.startState = startState
+
+    @property
+    def name(self):
+        if self.configSpec.operation:
+            name = self.configSpec.operation
+        else:
+            name = self.configSpec.name
+        if self.reason and self.reason not in name:
+            return name + " (reason: " + self.reason + ")"
+        return name
+
+    def __repr__(self):
+        return "TaskRequest(%s(%s,%s):%s)" % (
+            self.target,
+            self.target.status,
+            self.target.state,
+            self.name,
+        )
+
+
+class SetStateRequest(PlanRequest):
+    def __init__(self, target, state):
+        super(SetStateRequest, self).__init__(target)
+        self.set_state = state
+
+    @property
+    def name(self):
+        return self.set_state
+
+
+class TaskRequestGroup(PlanRequest):
+    def __init__(self, target, workflow):
+        super(TaskRequestGroup, self).__init__(target)
+        self.workflow = workflow
+        self.children = []
+
+    def __repr__(self):
+        return "TaskRequestGroup(%s:%s:%s)" % (
+            self.target,
+            self.workflow,
+            self.children,
+        )
 
 
 class JobRequest(object):
@@ -230,6 +286,8 @@ class Configurator(object):
     as an alternative to using the full name ("module.class") when setting the implementation on an operation.
     (Titlecase recommended)"""
 
+    excludeFromDigest = ()
+
     def __init__(self, configurationSpec):
         self.configSpec = configurationSpec
 
@@ -283,10 +341,56 @@ class Configurator(object):
         """Does this configuration need to be run?"""
         return self.configSpec.shouldRun()
 
-    # XXX3 should be called during when checking dependencies
-    # def checkConfigurationStatus(self, task):
-    #   """Is this configuration still valid?"""
-    #   return Status.ok
+    def saveDigest(self, task):
+        """
+        Generate a compact, deterministic representation of the current configuration.
+        This is saved in the job log and used by `checkDigest` in subsequent jobs to
+        determine if the configuration changed the operation needs to be re-run.
+
+        The default implementation calculates a SHA1 digest of the values of the inputs
+        that where accessed while that task was run, with the exception of
+        the input parameters listed in `excludeFromDigest`.
+
+        Args:
+            task (:class:`TaskView`) The task that executed this operation.
+
+        Returns:
+            dict: A dictionary whose keys are strings that start with "digest"
+        """
+        # XXX user definition should be able to exclude inputs from digest
+        inputs = task.inputs.getResolved()
+        keys = sorted([k for k in inputs.keys() if k not in self.excludeFromDigest])
+        if keys:
+            inputdigest = getDigest([inputs[key] for key in keys])
+        else:
+            inputdigest = ""
+        return dict(digestKeys=",".join(keys), digestValue=inputdigest)
+
+    def checkDigest(self, task, changeset):
+        """
+        Generate a compact, deterministic representation of the current configuration.
+        This is saved in the job log and used by `checkDigest` in subsequent jobs to
+        determine if the configuration changed the operation needs to be re-run.
+
+        The default implementation recalculates the digest of input parameters that
+        were accessed in the previous run.
+
+        Args:
+            task (:class:`TaskView`) The task that might execute this operation.
+
+        Returns:
+            bool: True if configuration's digest has changed, false if it the same.
+        """
+        _parameters = changeset.digestKeys
+        if not _parameters:
+            return False
+        keys = _parameters.split(",")
+        # an old input was removed
+        if set(keys) - set(task.inputs.keys()):
+            return True
+        # only resolve the inputs that were resolved before
+        values = [task.inputs[key] for key in keys]
+        return changeset.digestValue != getDigest(values)
 
 
 class TaskView(object):
@@ -307,7 +411,7 @@ class TaskView(object):
         self.messages = []
         self._addedResources = []
         self._dependenciesChanged = False
-        self.dependencies = dependencies or {}
+        self.dependencies = dependencies or []
         self._resourceChanges = ResourceChanges()
         # public:
         self.operationHost = self._findOperationHost(target, configSpec.operationHost)
@@ -337,6 +441,8 @@ class TaskView(object):
                 task=self.getSettings(),
                 connections=list(self._getConnections()),
                 allConnections=self._getAllConnections(),
+                TOPOLOGY=dict(inputs=target.root.inputs._attributes),
+                NODES=TopologyMap(target.root),
                 SELF=self.target.attributes,
                 HOST=HOST,
                 ORCHESTRATOR=ORCHESTRATOR and ORCHESTRATOR.attributes or {},
@@ -577,6 +683,7 @@ class TaskView(object):
         resolveExternal=True,
         strict=True,
         vars=None,
+        throw=False,
     ):
         # XXX pass resolveExternal to context?
         try:
@@ -584,10 +691,12 @@ class TaskView(object):
                 self.inputs.context, wantList, strict
             )
         except:
-            UnfurlTaskError(
-                self, "error while evaluating query: %s" % query, logging.WARNING
-            )
-            return None
+            if not throw:
+                UnfurlTaskError(
+                    self, "error while evaluating query: %s" % query, logging.WARNING
+                )
+                return None
+            raise
 
         if dependency:
             self.addDependency(
@@ -610,7 +719,7 @@ class TaskView(object):
             expr = Ref(getter()).source
 
         dependency = Dependency(expr, expected, schema, name, required, wantList)
-        self.dependencies[name or expr] = dependency
+        self.dependencies.append(dependency)
         self.dependenciesChanged = True
         return dependency
 
@@ -700,18 +809,18 @@ class TaskView(object):
             try:
                 resources = yaml.load(resources)
             except:
-                UnfurlTaskError(self, "unable to parse as YAML: %s" % resources)
-                return None
+                err = UnfurlTaskError(self, "unable to parse as YAML: %s" % resources)
+                return None, [err]
 
         if isinstance(resources, collections.Mapping):
             resources = [resources]
         elif not isinstance(resources, collections.MutableSequence):
-            UnfurlTaskError(
+            err = UnfurlTaskError(
                 self,
                 "updateResources requires a list of updates, not a %s"
                 % type(resources),
             )
-            return None
+            return None, [err]
 
         errors = []
         newResources = []
@@ -721,12 +830,13 @@ class TaskView(object):
             if not isinstance(resourceSpec, collections.Mapping):
                 continue
             originalResourceSpec = resourceSpec
+            rname = resourceSpec.get("name", "SELF")
+            if rname == ".self" or rname == "SELF":
+                existingResource = self.target
+                rname = existingResource.name
+            else:
+                existingResource = self.findInstance(rname)
             try:
-                rname = resourceSpec.get("name", "SELF")
-                if rname == ".self" or rname == "SELF":
-                    existingResource = self.target
-                else:
-                    existingResource = self.findInstance(rname)
 
                 if existingResource:
                     updated = False
@@ -748,7 +858,7 @@ class TaskView(object):
                             attributes, existingResource
                         ).items():
                             existingResource.attributes[key] = value
-                            logger.debug(
+                            self.logger.debug(
                                 "setting attribute %s with %s on %s",
                                 key,
                                 value,
@@ -757,7 +867,7 @@ class TaskView(object):
                         updated = True
 
                     if updated:
-                        logger.info("updating resources %s", existingResource.name)
+                        self.logger.info("updating resources %s", existingResource.name)
                     continue
 
                 pname = resourceSpec.get("parent")
@@ -804,7 +914,9 @@ class TaskView(object):
                 # if resource.required or resourceSpec.get("dependent"):
                 #    self.addDependency(resource, required=resource.required)
             except:
-                errors.append(UnfurlAddingResourceError(self, originalResourceSpec))
+                errors.append(
+                    UnfurlAddingResourceError(self, originalResourceSpec, rname)
+                )
             else:
                 newResourceSpecs.append(originalResourceSpec)
                 newResources.append(resource)
@@ -812,16 +924,16 @@ class TaskView(object):
         if newResourceSpecs:
             self._resourceChanges.addResources(newResourceSpecs)
             self._addedResources.extend(newResources)
-            logger.info("add resources %s", newResources)
+            self.logger.info("add resources %s", newResources)
 
             jobRequest = JobRequest(newResources, errors)
             if self.job:
                 self.job.jobRequestQueue.append(jobRequest)
-            return jobRequest
-        return None
+            return jobRequest, errors
+        return None, errors
 
 
-class Dependency(ChangeAware):
+class Dependency(Operational):
     """Represents a runtime dependency for a configuration.
 
 
@@ -855,10 +967,18 @@ class Dependency(ChangeAware):
 
         self.expected = expected
         self.schema = schema
-        self.required = required
+        self._required = required
         self.name = name
         self.wantList = wantList
 
+    @property
+    def localStatus(self):
+        return Status.ok
+
+    @property
+    def priority(self):
+        return Priority.required if self._required else Priority.optional
+
     def refresh(self, config):
         if self.expected is not None:
             changeId = config.changeId
