@@ -88,6 +88,7 @@ def _truncate(s):
 
 # XXX we should know if cmd if not os.access(implementation, os.X):
 class ShellConfigurator(TemplateConfigurator):
+    excludeFromDigest = TemplateConfigurator.excludeFromDigest + ("cwd", "echo")
     _defaultCmd = None
 
     @staticmethod
@@ -174,10 +175,10 @@ class ShellConfigurator(TemplateConfigurator):
             err.error = err
             return err
 
-    def _handleResult(self, task, result):
-        error = result.error or result.returncode or result.timeout
+    def _handleResult(self, task, result, cwd, successCodes=(0,)):
+        error = result.error or result.returncode not in successCodes or result.timeout
         if error:
-            task.logger.warning("shell task run failure: %s", result.cmd)
+            task.logger.warning('shell task run failure: "%s" in %s', result.cmd, cwd)
             if result.error:
                 task.logger.info("shell task error", exc_info=result.error)
             else:
@@ -194,8 +195,8 @@ class ShellConfigurator(TemplateConfigurator):
         result.stderr = unstyle(result.stderr or "")
         return not error
 
-    def _processResult(self, task, result):
-        success = self._handleResult(task, result)
+    def _processResult(self, task, result, cwd):
+        success = self._handleResult(task, result, cwd)
         resultDict = result.__dict__.copy()
         resultDict["success"] = success
         self.processResultTemplate(task, resultDict)
@@ -226,31 +227,14 @@ class ShellConfigurator(TemplateConfigurator):
         cwd = params.get("cwd")
         if cwd:
             # if cwd is relative, make it relative to task.cwd
-            assert isinstance(cmd, six.string_types)
+            assert isinstance(cwd, six.string_types)
             cwd = os.path.abspath(os.path.join(task.cwd, cwd))
         else:
             cwd = task.cwd
         keeplines = params.get("keeplines")
 
-        if task.dryRun and isinstance(task.inputs.get("dryrun"), six.string_types):
-            dryrunArg = task.inputs["dryrun"]
-            if "%dryrun%" in cmd:  # replace %dryrun%
-                if isString:
-                    cmd = cmd.replace("%dryrun%", dryrunArg)
-                else:
-                    cmd[cmd.index("%dryrun%")] = dryrunArg
-            else:  # append dryrunArg
-                if isString:
-                    cmd += " " + dryrunArg
-                else:
-                    cmd.append(dryrunArg)
-        elif "%dryrun%" in cmd:
-            if isString:
-                cmd = cmd.replace("%dryrun%", "")
-            else:
-                cmd.remove("%dryrun%")
-
-        echo = params.get("echo")  # task.verbose > -1)
+        cmd = self.resolve_dry_run(cmd, task)
+        echo = params.get("echo",  task.verbose > -1)
         result = self.runProcess(
             cmd,
             shell=shell,
@@ -260,7 +244,28 @@ class ShellConfigurator(TemplateConfigurator):
             keeplines=keeplines,
             echo=echo,
         )
-        success = self._processResult(task, result)
+        success = self._processResult(task, result, cwd)
         done = task.inputs.get("done", {})
         success = done.pop("success", success)
         yield task.done(success, result=result.__dict__, **done)
+
+    def resolve_dry_run(self, cmd, task):
+        is_string = isinstance(cmd, six.string_types)
+        if task.dryRun and isinstance(task.inputs.get("dryrun"), six.string_types):
+            dry_run_arg = task.inputs["dryrun"]
+            if "%dryrun%" in cmd:  # replace %dryrun%
+                if is_string:
+                    cmd = cmd.replace("%dryrun%", dry_run_arg)
+                else:
+                    cmd[cmd.index("%dryrun%")] = dry_run_arg
+            else:  # append dry_run_arg
+                if is_string:
+                    cmd += " " + dry_run_arg
+                else:
+                    cmd.append(dry_run_arg)
+        elif "%dryrun%" in cmd:
+            if is_string:
+                cmd = cmd.replace("%dryrun%", "")
+            else:
+                cmd.remove("%dryrun%")
+        return cmd
