@@ -79,7 +79,8 @@ def markSensitive(schemas, state, task, sensitiveNames=()):
 class TerraformConfigurator(ShellConfigurator):
     _defaultCmd = "terraform"
 
-    sensitiveNames = ["access_token", "key_material", "password"]
+    # provider schemas don't always mark keys as sensitive that they should, so just in case:
+    sensitiveNames = ["access_token", "key_material", "password", "private_key"]
 
     def canDryRun(self, task):
         return True
@@ -89,12 +90,17 @@ class TerraformConfigurator(ShellConfigurator):
         timeout = task.configSpec.timeout
         cmd = terraform + ["init"]
         result = self.runProcess(cmd, timeout=timeout, env=env, cwd=cwd, echo=echo)
-        if not self._handleResult(task, result):
+        if not self._handleResult(task, result, cwd):
             return None
 
         cmd = terraform + "providers schema -json".split(" ")
         result = self.runProcess(cmd, timeout=timeout, env=env, cwd=cwd, echo=False)
-        if not self._handleResult(task, result):
+        if not self._handleResult(task, result, cwd):
+            task.logger.warning(
+                "terraform providers schema failed: %s %s",
+                result.returncode,
+                result.stderr,
+            )
             return None
 
         try:
@@ -114,7 +120,6 @@ class TerraformConfigurator(ShellConfigurator):
             Write out tf.json if necessary.
         """
         # generated tf.json get written to as main.unfurl.tmp.tf.json
-        # (add *.tmp.* to git ignore)
         main = task.inputs.getCopy("main")
         if main:
             if isinstance(main, six.string_types):
@@ -133,11 +138,11 @@ class TerraformConfigurator(ShellConfigurator):
         if tfvars:
             if isinstance(tfvars, six.string_types):
                 # assume the contents of a tfvars file
-                path = "vars.tfvars"
+                path = "vars.tmp.tfvars"
             else:
-                path = "vars.tfvars.json"
+                path = "vars.tmp.tfvars.json"
             ctx = task.inputs.context
-            return writeFile(ctx, tfvars, path, "local")
+            return writeFile(ctx, tfvars, path, "home")
         return None
 
     def _prepareState(self, task, ctx):
@@ -145,7 +150,7 @@ class TerraformConfigurator(ShellConfigurator):
         # read the (possible encrypted) version from the repository
         # and write out it as plaintext json into the local directory
         yamlPath = abspath(ctx, "terraform.tfstate.yaml", "home", False)
-        jsonPath = abspath(ctx, "terraform.tfstate.json", "local", False)
+        jsonPath = abspath(ctx, "terraform.tfstate.json", "home", False)
         if not os.path.exists(jsonPath) and os.path.exists(yamlPath):
             # if exists in home, load and write out state file as json
             with open(yamlPath, "r") as f:
@@ -158,7 +163,7 @@ class TerraformConfigurator(ShellConfigurator):
         # read the (possible encrypted) version from the repository
         # and write out it as plaintext json into the local directory
         jobId = task.getJobId(task.changeId)
-        return abspath(ctx, jobId + ".plan", "local", False)
+        return abspath(ctx, jobId + ".plan", "local")
 
     def run(self, task):
         ctx = task.inputs.context
@@ -167,7 +172,7 @@ class TerraformConfigurator(ShellConfigurator):
         _, terraform = self._cmd(
             task.inputs.get("command", self._defaultCmd), task.inputs.get("keeplines")
         )
-        cwd = os.path.abspath(task.inputs.get("dir") or getdir(ctx, "spec.home"))
+        cwd = os.path.abspath(task.inputs.get("dir") or getdir(ctx, "home"))
         dataDir = os.getenv("TF_DATA_DIR", os.path.join(cwd, ".terraform"))
 
         # write out any needed files to cwd, eg. main.tf.json
@@ -198,6 +203,7 @@ class TerraformConfigurator(ShellConfigurator):
                 "plan",
                 "-state=" + statePath,
                 "-detailed-exitcode",
+                "-refresh=true",
                 "-out",
                 planPath,
             ]
@@ -207,7 +213,7 @@ class TerraformConfigurator(ShellConfigurator):
             action = ["destroy", "-auto-approve", "-state=" + statePath]
         elif task.configSpec.workflow == "deploy":
             action = ["apply", "-auto-approve", "-state=" + statePath]
-            if os.path.isfile(planPath):
+            if os.path.isfile(planPath) and os.path.isfile(statePath):
                 action.append(
                     planPath
                 )  # use plan created by previous operation in this job
@@ -236,13 +242,16 @@ class TerraformConfigurator(ShellConfigurator):
 
         # process the result
         status = None
-        success = self._handleResult(task, result)
+        success = self._handleResult(task, result, cwd, (0, 2))
         if result.returncode == 2:
             # plan -detailed-exitcode: 2 - Succeeded, but there is a diff
             success = True
-            status = Status.ok
+            if task.configSpec.operation == "check":
+                status = Status.absent
+            else:
+                status = Status.ok
 
-        if success and not task.dryRun:
+        if success and not (task.dryRun or task.configSpec.operation == "check"):
             # read state file
             with open(statePath) as sf:
                 state = json.load(sf)
