@@ -466,7 +466,11 @@ def applyTemplate(value, ctx, overrides=None):
         try:
             value = templar.template(value, fail_on_undefined=fail_on_undefined)
         except Exception as e:
-            value = "<<Error rendering template: %s>>" % str(e)
+            msg = str(e)
+            match = re.search(r"has no attribute '(\w+)'", msg)
+            if match:
+                msg = 'missing attribute or key: "%s"' % match.group(1)
+            value = "<<Error rendering template: %s>>" % msg
             if ctx.strict:
                 logger.debug(value, exc_info=True)
                 raise UnfurlError(value)
@@ -954,6 +958,25 @@ class ResourceChanges(collections.OrderedDict):
         self.clear()
 
 
+class TopologyMap(dict):
+    # need to subtype dict directly to make jinja2 happy
+    def __init__(self, resource):
+        self.resource = resource
+
+    def __getitem__(self, key):
+        r = self.resource.findResource(key)
+        if r:
+            return r.attributes
+        else:
+            raise KeyError(key)
+
+    def __iter__(self):
+        return iter(r.name for r in self.resource.getSelfAndDescendents())
+
+    def __len__(self):
+        return len(tuple(self.resource.getSelfAndDescendents()))
+
+
 class AttributeManager(object):
     """
     Tracks changes made to Resources
@@ -998,15 +1021,20 @@ class AttributeManager(object):
     def getAttributes(self, resource):
         if resource.key not in self.attributes:
             if resource.template:
-                specd = resource.template.properties
-                defaultAttributes = resource.template.defaultAttributes
+                specAttributes = resource.template.defaultAttributes
                 _attributes = ChainMap(
-                    copy.deepcopy(resource._attributes), specd, defaultAttributes
+                    copy.deepcopy(resource._attributes),
+                    resource.template.properties,
+                    specAttributes,
                 )
             else:
                 _attributes = ChainMap(copy.deepcopy(resource._attributes))
 
-            attributes = ResultsMap(_attributes, RefContext(resource))
+            vars = dict(NODES=TopologyMap(resource.root))
+            if hasattr(resource.root, "inputs"):
+                vars["TOPOLOGY"] = dict(inputs=resource.root.inputs._attributes)
+            ctx = RefContext(resource, vars)
+            attributes = ResultsMap(_attributes, ctx)
             self.attributes[resource.key] = (resource, attributes)
             return attributes
         else:
@@ -1017,38 +1045,50 @@ class AttributeManager(object):
     #   # for resource, old, new in self.statuses.values():
     #   #   resource._localStatus = old
 
+    def _saveSensitive(self, defs, key, value, resource):
+        defSchema = (key in defs and defs[key].schema) or {}
+        defMeta = defSchema.get("metadata", {})
+        # attribute marked as sensitive and value isn't a secret so mark value as sensitive
+        # but externalvalues are ok since they don't reveal much
+        sensitive = defMeta.get("sensitive") and not value.external
+        if sensitive:
+            savedValue = wrapSensitiveValue(
+                value.resolved, resource.templar._loader._vault
+            )
+        else:
+            savedValue = value.asRef()  # serialize Result
+        return savedValue, sensitive
+
     def commitChanges(self):
         changes = {}
+        liveDependencies = {}
         for resource, attributes in self.attributes.values():
             # save in _attributes in serialized form
             overrides, specd = attributes._attributes.split()
             resource._attributes = {}
             defs = resource.template and resource.template.attributeDefs or {}
             foundSensitive = []
+            live = {}
             for key, value in overrides.items():
                 if not isinstance(value, Result):
                     # hasn't been touched so keep it as is
                     resource._attributes[key] = value
-                elif key not in specd or value.hasDiff():
+                else:
                     # value is a Result and it is either new or different from the original value, so save
-                    defSchema = (key in defs and defs[key].schema) or {}
-                    defMeta = defSchema.get("metadata", {})
-
-                    # XXX if defMeta.get('immutable') and key in specd:
-                    #  error('value of attribute "%s" changed but is marked immutable' % key)
-
-                    if defMeta.get("sensitive"):
-                        # attribute marked as sensitive and value isn't a secret so mark value as sensitive
-                        if (
-                            not value.external
-                        ):  # externalvalues are ok since they don't reveal much
-                            # we won't be able to restore this value since we can't save it
-                            resource._attributes[key] = wrapSensitiveValue(
-                                value.resolved, resource.templar._loader._vault
-                            )
+                    changed = key not in specd or value.hasDiff()
+                    isLive = key in resource.template.defaultAttributes
+                    if not (changed or isLive):
+                        continue
+                    savedValue, sensitive = self._saveSensitive(
+                        defs, key, value, resource
+                    )
+                    live[key] = savedValue
+                    if changed:
+                        if sensitive:
                             foundSensitive.append(key)
-                            continue
-                    resource._attributes[key] = value.asRef()  # serialize Result
+                        # XXX if defMeta.get('immutable') and key in specd:
+                        #  error('value of attribute "%s" changed but is marked immutable' % key)
+                        resource._attributes[key] = savedValue
 
             # save changes
             diff = attributes.getDiff()
@@ -1058,7 +1098,9 @@ class AttributeManager(object):
                 if key in diff:
                     diff[key] = resource._attributes[key]
             changes[resource.key] = diff
+            if live:
+                liveDependencies[resource.key] = live
 
         self.attributes = {}
         # self.statuses = {}
-        return changes
+        return changes, liveDependencies
