@@ -113,11 +113,8 @@ class UnfurlTaskError(UnfurlError):
 
 
 class UnfurlAddingResourceError(UnfurlTaskError):
-    def __init__(self, task, resourceSpec, log=logging.WARNING):
-        resourcename = isinstance(resourceSpec, Mapping) and resourceSpec.get(
-            "name", ""
-        )
-        message = "error creating resource %s" % resourcename
+    def __init__(self, task, resourceSpec, name, log=logging.DEBUG):
+        message = "error updating resource %s" % name
         super(UnfurlAddingResourceError, self).__init__(task, message, log)
         self.resourceSpec = resourceSpec
 
@@ -279,7 +276,7 @@ def toEnum(enum, value, default=None):
         return value
 
 
-def _dump(obj, tp, suffix="", yaml=None, encoding=None):
+def dump(obj, tp, suffix="", yaml=None, encoding=None):
     from .yamlloader import yaml as _yaml
 
     try:
@@ -312,7 +309,7 @@ def _dump(obj, tp, suffix="", yaml=None, encoding=None):
 
         # try to dump any other object as json
         if obj is not None:  # treat None as 0 byte file
-            json.dump(obj, f, indent=2)
+            json.dump(obj, f, indent=2, sort_keys=True)
     finally:
         if six.PY3:
             f.detach()
@@ -325,7 +322,7 @@ def _saveToVault(path, obj, yaml=None, encoding=None, fd=None):
         vault = VaultEditor(yaml.representer.vault)
         f = io.BytesIO()
         vpath = path[: -len(".vault")] if vaultExt else path
-        _dump(obj, f, vpath, yaml, encoding)
+        dump(obj, f, vpath, yaml, encoding)
         b_vaulttext = yaml.representer.vault.encrypt(f.getvalue())
         vault.write_data(b_vaulttext, fd or path)
         return True
@@ -338,7 +335,7 @@ def saveToFile(path, obj, yaml=None, encoding=None):
         os.makedirs(dir)
     if not _saveToVault(path, obj, yaml, encoding):
         with open(path, "wb") as f:
-            _dump(obj, f, path, yaml, encoding)
+            dump(obj, f, path, yaml, encoding)
 
 
 def saveToTempfile(obj, suffix="", delete=True, dir=None, yaml=None, encoding=None):
@@ -353,7 +350,7 @@ def saveToTempfile(obj, suffix="", delete=True, dir=None, yaml=None, encoding=No
 
     if not _saveToVault(suffix or "", obj, yaml, encoding, tp.fileno()):
         try:
-            _dump(obj, tp, suffix or "", yaml, encoding)
+            dump(obj, tp, suffix or "", yaml, encoding)
         finally:
             tp.close()
     return tp
@@ -436,8 +433,10 @@ class ChainMap(Mapping):
     Combine multiple mappings for sequential lookup.
     """
 
-    def __init__(self, *maps):
+    def __init__(self, *maps, **kw):
         self._maps = maps
+        self._track = kw.get("track")
+        self.accessed = set()
 
     def split(self):
         return self._maps[0], ChainMap(*self._maps[1:])
@@ -448,6 +447,9 @@ class ChainMap(Mapping):
                 return mapping[key]
             except KeyError:
                 pass
+            else:
+                if self._track is mapping:
+                    self.accessed.add(key)
         raise KeyError(key)
 
     def __setitem__(self, key, value):
