@@ -31,6 +31,7 @@ from .util import (
     UnfurlError,
     UnfurlValidationError,
     findSchemaErrors,
+    getBaseDir,
 )
 from .merge import (
     expandDoc,
@@ -348,7 +349,7 @@ class YamlConfig(object):
 
     def getBaseDir(self):
         if self.path:
-            return os.path.dirname(self.path)
+            return getBaseDir(self.path)
         else:
             return "."
 
@@ -426,7 +427,22 @@ class YamlConfig(object):
         with open(path, "w") as f:
             f.write(output.getvalue())
 
-    def loadInclude(self, templatePath, warnWhenNotFound=False):
+    def loadInclude(
+        self, templatePath, warnWhenNotFound=False, expanded=None, check=False
+    ):
+        if check:
+            if self.loadHook:
+                return self.loadHook(
+                    self,
+                    templatePath,
+                    self.baseDirs[-1],
+                    warnWhenNotFound,
+                    expanded,
+                    check,
+                )
+            else:
+                return True
+
         if templatePath == self.baseDirs[-1]:  # pop hack
             self.baseDirs.pop()
             return
@@ -452,7 +468,7 @@ class YamlConfig(object):
             baseDir = self.baseDirs[-1]
             if self.loadHook:
                 path, template = self.loadHook(
-                    self, templatePath, baseDir, warnWhenNotFound
+                    self, templatePath, baseDir, warnWhenNotFound, expanded
                 )
             else:
                 path, template = self.loadYaml(key, baseDir, warnWhenNotFound)
